@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: ConfigDefs.pl 4522 2008-08-20 15:19:23Z sysjkf $
+#   $Id: ConfigDefs.pl 4713 2009-04-01 16:07:04Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -50,6 +50,14 @@ __DATA__
 
 [Translation,Translation]
 
+AFilenameRules			= ArchivesFilenameRules
+AFiletypeRules			= ArchivesFiletypeRules
+aallowfilenames			= ArchivesAllowFilenames
+adenyfilemimetypes		= ArchivesDenyFileMIMETypes
+adenyfiletypes			= ArchivesDenyFiletypes
+aallowfilemimetypes		= ArchivesAllowFileMIMETypes
+aallowfiletypes			= ArchivesAllowFiletypes
+adenyfilenames			= ArchivesDenyFilenames
 addenvfrom			= AddEnvelopeFromHeader
 addenvto			= AddEnvelopeToHeader
 addmshmac			= AddWatermark
@@ -135,6 +143,7 @@ keepspamarchiveclean		= keepspamandmcparchiveclean
 lastafterbatch			= alwayslookeduplastafterbatch
 lastlookup			= alwayslookeduplast
 listsascores                    = includescoresinspamassassinreport
+logdelivery			= logdeliveryandnondelivery
 loghtmltags			= logdangeroushtmltags
 logfacility			= syslogfacility
 logsaactions			= logspamassassinruleactions
@@ -158,6 +167,7 @@ mshmacnull			= TreatInvalidWatermarksWithNoSenderAsSpam
 mshmacvalid			= WatermarkLifetime
 namemodifysubject		= filenamemodifysubject
 namesubjecttext			= filenamesubjecttext
+newheadersattop			= placenewheadersattopofmessage
 noisyviruses			= nonforgingviruses
 normalrbls                      = spamliststobespam
 nosenderprecedence		= nevernotifysendersofprecedence
@@ -168,6 +178,8 @@ phishinghighlight		= highlightphishingfraud
 phishingnumbers			= alsofindnumericphishing
 phishingsubjecttag		= phishingsubjecttext
 phishingwhitelist		= phishingsafesitesfile
+procdbattempts			= MaximumProcessingAttempts
+procdbname			= ProcessingAttemptsDatabase
 outqueuedir			= outgoingqueuedir
 quarantinesilent		= quarantinesilentviruses
 quarantineuser			= quarantineuser
@@ -226,6 +238,7 @@ mcplistsascores			= includescoresinmcpreport
 mcpreqspamassassinscore		= mcprequiredspamassassinscore
 mcpwhitelist			= isdefinitelynotmcp
 syntaxcheck			= automaticsyntaxcheck
+unpackole			= UnpackMicrosoftDocuments
 unscannedheader			= unscannedheadervalue
 usedefaultswithmanyrecips       = usedefaultruleswithmultiplerecipients
 tagphishingsubject		= phishingmodifysubject
@@ -263,6 +276,7 @@ debug			0	no	0	yes	1
 debugspamassassin	0	no	0	yes	1
 deliverinbackground	1	no	0	yes	1
 getipfromheader		0	no	0	yes	1
+logdelivery		0	no	0	yes	1
 lognonspam		0	no	0	yes	1
 logsaactions		0	no	0	yes	1
 logsilentviruses	0	no	0	yes	1
@@ -282,7 +296,7 @@ usedefaultswithmanyrecips	0	no	0	yes	1
 # These should be checked for dir existence
 [Simple,Dir]
 incomingworkdir		/var/spool/MailScanner/incoming
-lockfiledir		/tmp
+lockfiledir		/var/spool/MailScanner/incoming/Locks
 
 # Check the first word of these for file existence
 [Simple,File]
@@ -318,7 +332,7 @@ MaxDirtyMessages		30
 MaxGSSize			20000
 MaxGSTimeouts			10
 MaxSpamAssassinTimeouts		10
-MaxSpamCheckSize		150000
+ProcDBAttempts			6
 QueueScanInterval		6
 RBLTimeoutLen			10
 RestartEvery			14400
@@ -357,6 +371,7 @@ MailScannerVersionNumber	1.0.0
 MaxSpamAssassinSize		30000
 MinimumCodeStatus	supported
 MTA			sendmail
+ProcDBName		/var/spool/MailScanner/incoming/Processing.db
 QuarantineUser
 QuarantineGroup
 QuarantinePerms		0600
@@ -448,6 +463,7 @@ MCPChecks		0	no	0	yes	1
 MCPDetail		1	no	0	yes	1
 MCPListSAScores		0	no	0	yes	1
 MCPWhitelist		0	no	0	yes	1
+UnpackOle		1	no	0	yes	1
 UseSACache		1	no	0	yes	1
 VirusModifySubject	start	no	0	yes	1	start	start	end	end
 warningisattachment	1	no	0	yes	1
@@ -503,6 +519,7 @@ HighSpamAssassinScore		10
 MaxAttachmentSize		-1
 MaxMessageSize			0
 MaxParts			200
+MaxSpamCheckSize		150000
 MaxSpamListTimeouts		7
 MaxZipDepth			2
 MCPErrorScore			1
@@ -516,6 +533,7 @@ ReqSpamAssassinScore		6
 
 [First,Other]
 Antiword			/usr/bin/antiword -f
+ArchivesAre			zip rar uu ole tnef
 AttachmentCharset		ISO-8859-1
 attachimageinternalname
 attachimagename
@@ -589,6 +607,7 @@ FindPhishing		1	no	0	yes	1
 markinfectedmessages	1	no	0	yes	1
 PhishingHighlight	1	no	0	yes	1
 HtmlToText		0	no	0	yes	1
+NewHeadersAtTop		0	no	0	yes	1
 PhishingNumbers		1	no	0	yes	1
 QuarantineInfections	1	no	0	yes	1
 QuarantineModifiedBody	0	no	0	yes	1
@@ -613,6 +632,14 @@ ZipAttachments		0	no	0	yes	1
 [All,Other]
 # This is the other stuff that came up in the search that I haven't
 # figured out what to do with yet...
+AFilenameRules
+AFiletypeRules
+aallowfilenames
+adenyfilemimetypes
+adenyfiletypes
+aallowfilemimetypes
+aallowfiletypes
+adenyfilenames
 ArchiveMail
 AttachZipIgnore			.zip .rar .gz .tgz .mpg .mpe .mpeg .mp3 .rpm
 ClamdLockFile
