@@ -246,13 +246,14 @@ sub LFileName {
 #  }
 
 sub ReadQf {
-  my($this, $message) = @_;
+  my($this, $message, $getipfromheader) = @_;
 
   my($RQf) = $message->{store}{inhhandle};
 
   my %metadata;
   my($InHeader, $InSubject, $InDel, @headers, $msginfo, $from, @to, $subject);
-  my($ip, $sender, @acl, @aclc, @aclm, $line, $acltype);
+  my($ip, $sender, %acl, %aclc, %aclm, $line, $acltype);
+  my($read1strcvd, $ipfromheader);
 
   #print STDERR "ReadQf for " . $message->{id} . "\n";
 
@@ -294,16 +295,16 @@ sub ReadQf {
     $acltype = $1;
     if($acltype =~ /^acl[cm]?$/) {
       # we need to handle acl vars differently
-      if($line =~ /^(\w+) (\d+)$/) {
+      if($line =~ /^(\w+|_[[:alnum:]_]+) (\d+)$/) {
         my $buf;
         my $pos = $1;
         my $len = $2;
         if ($acltype eq "acl") {
-          $acl[$pos]->[0] = [];
+          $acl{$pos}->[0] = [];
         } elsif ($acltype eq "aclc") {
-          $aclc[$pos]->[0] = [];
+          $aclc{$pos}->[0] = [];
         } elsif ($acltype eq "aclm") {
-          $aclm[$pos]->[0] = [];
+          $aclm{$pos}->[0] = [];
         } else {
           # invalid format
           last;
@@ -316,11 +317,11 @@ sub ReadQf {
           last;
         }
         if ($acltype eq "acl") {
-          $acl[$pos]->[0] = $buf;
+          $acl{$pos}->[0] = $buf;
         } elsif ($acltype eq "aclc") {
-          $aclc[$pos]->[0] = $buf;
+          $aclc{$pos}->[0] = $buf;
         } elsif ($acltype eq "aclm") {
-          $aclm[$pos]->[0] = $buf;
+          $aclm{$pos}->[0] = $buf;
         } else {
           # invalid format
           last;
@@ -337,9 +338,9 @@ sub ReadQf {
     }
     next;
   }
-  $metadata{aclvars} = \@acl;
-  $metadata{aclcvars} = \@aclc;
-  $metadata{aclmvars} = \@aclm;
+  $metadata{aclvars} = \%acl;
+  $metadata{aclcvars} = \%aclc;
+  $metadata{aclmvars} = \%aclm;
 
   # If it was an invalid queue file, log a warning and tell caller
   unless (defined $line) {
@@ -579,6 +580,20 @@ sub ReadQf {
 	  chomp($message->{subject} = $2);
 	  $InSubject = 1;
 	}
+        if ("received:" eq lc $1) {
+          my $received = $2;
+          if ($received =~ /\[(\d+\.\d+\.\d+\.\d+)\]/i) {
+            unless ($read1strcvd) {
+              $ipfromheader = $1;
+              $read1strcvd = 1;
+            }
+          } elsif ($received =~ /\[([\dabcdef.:]+)\]/i) {
+            unless ($read1strcvd) {
+              $ipfromheader = $1;
+              $read1strcvd = 1;
+            }
+          }
+        }
       }
       # Track anything we may be in the middle of
       $InDel = ($InDel && $InHeader);
@@ -593,6 +608,10 @@ sub ReadQf {
 		 "- continuing anyway.");
   }
 
+  # If we were told to read the IP from the header and it was there...
+  $message->{clientip} = $ipfromheader
+    if $getipfromheader && $read1strcvd && $ipfromheader ne "";
+
   # We should have the last header built but not pushed
   # onto the metadata headers array at this point...
   exists $header->{name} and push @{$metadata{headers}},$header;
@@ -797,7 +816,12 @@ sub AddHeader {
   $newvalue =~ s/\n*\Z/\n/;
 
   $newheader = { name => $newkey, body => $newvalue, flag => " " };
-  push @{$message->{metadata}{headers}}, $newheader;
+  # DKIM: Add header at top if adding headers at top
+  if ($message->{newheadersattop}) {
+    unshift @{$message->{metadata}{headers}}, $newheader;
+  } else {
+    push @{$message->{metadata}{headers}}, $newheader;
+  }
 
   return 1;
 }
@@ -811,13 +835,24 @@ sub AddHeader {
 sub DeleteHeader {
   my($this, $message, $key) = @_;
 
+  my $usingregexp = ($key =~ s/^\/(.*)\/$/$1/)?1:0;
+
+  # Add a colon if they forgot it.
+  $key .= ':' unless $usingregexp || $key =~ /:$/;
+  # If it's not a regexp, then anchor it and sanitise it.
+  $key = quotemeta($key) unless $usingregexp;
+
   # Delete header by flagging it as deleted rather than by
   # actually deleting it; might help with debugging.
   # Also keep track of any flags that we've managed to "vanish".
-  my($hdrnum);
+  my($hdrnum, $line);
   my $metadata = $message->{metadata};
   for ($hdrnum=0; $hdrnum<@{$metadata->{headers}}; $hdrnum++) {
-    next unless lc $metadata->{headers}[$hdrnum]{name} eq lc $key;
+    # Skip if they are using a header name and it doesn't match
+    next if !$usingregexp && lc $metadata->{headers}[$hdrnum]{name} ne lc $key;
+    # Skip if they are using a regexp and it doesn't match
+    $line = $metadata->{headers}[$hdrnum]{name} . $metadata->{headers}[$hdrnum]{body};
+    next if $usingregexp && $line !~ /$key/i;
     # Have found the right line
     $metadata->{headers}[$hdrnum]{flag} ne " "
       and $metadata->{vanishedflags}{$metadata->{headers}[$hdrnum]{flag}} = 1;
@@ -855,7 +890,8 @@ sub UniqHeader {
 sub ReplaceHeader {
   my($this, $message, $key, $newvalue) = @_;
 
-  $this->DeleteHeader($message, $key);
+  # DKIM: Don't do DeleteHeader if adding all headers at top
+  $this->DeleteHeader($message, $key) unless $message->{dkimfriendly};
   $this->AddHeader($message, $key, $newvalue);
 
   return 1;
@@ -1127,24 +1163,25 @@ sub CreateQf {
 
   # ACLs patch starts here
   # Add the separate ACL Vars
-  my @acl  = @{$metadata->{aclvars}};
-  my @aclc = @{$metadata->{aclcvars}};
-  my @aclm = @{$metadata->{aclmvars}};
-  my $greatestacl = $#acl;
-  $greatestacl    = $#aclc if $#aclc > $greatestacl;
-  $greatestacl    = $#aclm if $#aclm > $greatestacl;
-  for($i=0; $i<=$greatestacl; $i++) {
-    if($acl[$i]) {
-      $Qfile .= "-acl " . $i . " " . length($acl[$i]->[0]) . "\n";
-      $Qfile .= $acl[$i]->[0] . "\n";
+  my %acl  = %{$metadata->{aclvars}};
+  my %aclc = %{$metadata->{aclcvars}};
+  my %aclm = %{$metadata->{aclmvars}};
+  foreach(keys %acl){
+    if($acl{$_}) {
+      $Qfile .= "-acl " . $_ . " " . length($acl{$_}->[0]) . "\n";
+      $Qfile .= $acl{$_}->[0] . "\n";
     }
-    if($aclc[$i]) {
-      $Qfile .= "-aclc " . $i . " " . length($aclc[$i]->[0]) . "\n";
-      $Qfile .= $aclc[$i]->[0] . "\n";
+  }
+  foreach(keys %aclc){
+    if($aclc{$_}) {
+      $Qfile .= "-aclc " . $_ . " " . length($aclc{$_}->[0]) . "\n";
+      $Qfile .= $aclc{$_}->[0] . "\n";
     }
-    if($aclm[$i]) {
-      $Qfile .= "-aclm " . $i . " " . length($aclm[$i]->[0]) . "\n";
-      $Qfile .= $aclm[$i]->[0] . "\n";
+  }
+  foreach(keys %aclm){
+    if($aclm{$_}) {
+      $Qfile .= "-aclm " . $_ . " " . length($aclm{$_}->[0]) . "\n";
+      $Qfile .= $aclm{$_}->[0] . "\n";
     }
   }
 
@@ -1397,6 +1434,7 @@ sub CreateBatch {
   my(%ModDate, $mta, $file, $tmpdate, $invalidfiles);
   my(@SortedFiles, $id, $newmessage, @queuedirnames);
   my($batchempty, $CriticalQueueSize, $headerfileumask);
+  my($getipfromheader);
 
   # Old code left over from single queue dir
   #$queuedirname = $global::MS->{inq}{dir};
@@ -1416,6 +1454,7 @@ sub CreateBatch {
   # If there are too many messages in the queue, start processing in
   # directory storage order instead of date order.
   $CriticalQueueSize = MailScanner::Config::Value('criticalqueuesize');
+  $getipfromheader = MailScanner::Config::Value('getipfromheader');
 
   # Set what we will need the umask to be
   $headerfileumask = $global::MS->{work}->{fileumask};
@@ -1501,6 +1540,7 @@ sub CreateBatch {
     }
 
     $batchempty = 1;
+    my $maxattempts = MailScanner::Config::Value('procdbattempts');
 
     # Keep going until end of dir or have reached every imposed limit. This
     # now processes the files oldest first to make for fairer queue cleanups.
@@ -1530,13 +1570,67 @@ sub CreateBatch {
       #print STDERR "Adding $id to batch\n";
 
       # Lock and read the qf file. Skip this message if the lock fails.
-      $newmessage = MailScanner::Message->new($id, $queuedirname);
+      $newmessage = MailScanner::Message->new($id, $queuedirname,
+                                              $getipfromheader);
       if ($newmessage eq 'INVALID') {
         $invalidfiles .= "$id ";
         next;
       }
       next unless $newmessage;
 
+      # JKF 20090301 Skip this message if $id has been scanned
+      # too many times.
+      # JKF 20090301 Read the number of times this message id has
+      # been processed. If over the limit, then ignore it.
+      # JKF 20090301 Should ideally make it in a batch of its own,
+      # and try that once or twice, *then* skip it. But let's try
+      # a simple version first.
+      # Just do a "next" if we want to skip the message.
+      if ($maxattempts) {
+        my $nexttime = time + 120 + int(rand(240)); # 4 +- 2 minutes
+        #my $nexttime = time + 10 + int(rand(20)); # 4 +- 2 minutes
+        my @attempts = $MailScanner::ProcDBH->selectrow_array(
+         "SELECT id,count,nexttime FROM processing WHERE (id='" . $id . "')");
+        #print STDERR "id       = \"$attempts[0]\"\n";
+        #print STDERR "count    = \"$attempts[1]\"\n";
+        #print STDERR "nexttime = \"$attempts[2]\"\n";
+        if (@attempts && $attempts[1]>=$maxattempts) {
+          MailScanner::Log::WarnLog("Warning: skipping message %s as it has been attempted too many times", $id);
+          # JKF 20090301 next;
+          # Instead of just skipping it, quarantine it and notify
+          # the local postmaster.
+          $newmessage->QuarantineDOS();
+          #print STDERR "Moving $attempts[0], $attempts[1], $attempts[2] into archive table\n";
+          $MailScanner::ProcDBH->do(
+            "DELETE FROM processing WHERE id='" . $id . "'");
+          $MailScanner::ProcDBH->do(
+            "INSERT INTO archive (id, count, nexttime) " .
+            "VALUES ('$attempts[0]', $attempts[1], $attempts[2])");
+        } elsif (defined $attempts[1]) {
+          # We have tried this message before
+          if (time>=$attempts[2]) {
+            # Time for next attempt has arrived
+            $MailScanner::ProcDBH->do(
+              "UPDATE processing SET count=count+1, nexttime=$nexttime " .
+              " WHERE id='" . $id . "'");
+            MailScanner::Log::InfoLog(
+              "Making attempt %d at processing message %s",
+              $attempts[1]+1, $id);
+            #print STDERR "Incremented $id\n";
+          } else {
+            # Not time for next attempt yet, so ignore the message
+            $newmessage->DropFromBatch();
+            next;
+          }
+        } else {
+          # We have never seen this message before
+          $MailScanner::ProcDBH->do(
+            "INSERT INTO processing (id, count, nexttime) " .
+            "VALUES ('" . $id . "', 1, $nexttime)");
+          #print STDERR "Inserted $id\n";
+        }
+      }
+
       $batch->{messages}{"$id"} = $newmessage;
       $newmessage->{mtime} = $mtime;
       $batchempty = 0;
