@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: PFDiskStore.pm 4359 2008-03-24 12:04:12Z sysjkf $
+#   $Id: PFDiskStore.pm 4692 2009-03-10 17:54:21Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ use MailScanner::Config;
 use vars qw($VERSION @DeletesPending);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4359 $, 10;
+$VERSION = substr q$Revision: 4692 $, 10;
 
 # List of pending delete operations so we can clear up properly when killed
 @DeletesPending = ();
@@ -59,7 +59,7 @@ $VERSION = substr q$Revision: 4359 $, 10;
 #use vars qw($VERSION);
 #
 #### The package version, both in 1.23 style *and* usable by MakeMaker:
-#$VERSION = substr q$Revision: 4359 $, 10;
+#$VERSION = substr q$Revision: 4692 $, 10;
 
 # Attributes are
 #
@@ -326,6 +326,7 @@ sub WriteHeader {
     # Quietly drop the data structures of this message.
     $message->{deleted} = 1;
     $message->{gonefromdisk} = 1; # Don't try to delete the original
+    $message->{abandoned} = 1; # Retry this message as it was ditched
     unlink $tfile; # Delete the new file from the queue
     MailScanner::Lock::unlockclose($Tf);
     return;
@@ -506,11 +507,19 @@ sub ReadBody {
   # Handle trackback -- This is the tricky one
   if ($configwords[1] =~ /tr[ua]/i) {
     #print STDERR "Trackback:\n";
-    while (${@{$body}}[scalar(@{$body})-1] !~ /^\s*$/) {
-      print "Line is ****" . ${@{$body}}[scalar(@{$body})-1] . "****\n";
+    my $i;
+    for ($i=(@{$body}-1); $i>=0; $i--) {
+      last if $body->[$i] =~ /^\s*$/i;
       pop @{$body};
-      #print STDERR ".";
     }
+    #my $bodysize = $#{@$body}-1;
+    #while (${@{$body}}[$bodysize+1] !~ /^\s*@/) { 
+    #while(${@{$body}}[$#{@$body}] !~ /^\s*$/) {
+      #print "Line is ****" . ${@{$body}}[scalar(@{$body})-1] . "****\n";
+    #  pop @{$body};
+      #print STDERR ".";
+    #}
+
     #print STDERR "\n";
     $b->Done();
     return;
