@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Postfix.pm 4372 2008-04-02 15:45:26Z sysjkf $
+#   $Id: Postfix.pm 4704 2009-03-15 13:33:58Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -42,7 +42,7 @@ use Encode;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4372 $, 10;
+$VERSION = substr q$Revision: 4704 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -278,7 +278,7 @@ sub new {
 
   # Change for V4: returns lower-case $from and @to
   sub ReadQf {
-    my($this, $message) = @_;
+    my($this, $message, $getipfromheader) = @_;
 
     my($RQf) = $message->{store}{inhdhandle};
 
@@ -296,6 +296,8 @@ sub new {
     my $MaxpRecPos = 0; # Max position where a p record might occur.
     my @npos = (); # save all p record positions, although we only
                    # look at/use the first and last.
+    my $ipfromheader = ''; # The IP address from the first Received: line
+    my $read1strcvd = 0; # Have we read the 1st Received: line?
 
     #print STDERR "In ReadQf\n";
     #$message->{store}->print();
@@ -357,6 +359,7 @@ sub new {
       $message->{deleted} = 1;
       $message->{gonefromdisk} = 1; # Don't try to delete the original
       $message->{store}->Unlock(); # Unlock it so other processes can pick it up
+      $message->{abandoned} = 1;
       return 0;
     }
 
@@ -436,7 +439,8 @@ sub new {
 	     @npos = ();
 	  } else {
 	    seek $RQf, $recdata+0, 0; # jump back or forward. we should chk this works.
-	    if ($recdata+0 < $npos[$#npos-1]) {
+            if ($recdata+0 < $MsgContSize+$DataOffset) {
+	    #20090216 if ($recdata+0 < $npos[$#npos-1]) {
 	       if ($recdata+0 >= $npos[0]) {
 		  MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
 		  $ErrorFound = 1;
@@ -505,7 +509,8 @@ sub new {
 	      @npos = ();
 	   } else {
 	     seek $RQf, $recdata+0, 0; # jump back or forward. we should chk this works.
-	     if ($recdata+0 < $npos[$#npos-1]) {
+             if ($recdata+0 < $MsgContSize+$DataOffset) {
+	     #20090216 if ($recdata+0 < $npos[$#npos-1]) {
 	        if ($recdata+0 >= $npos[0]) {
         	   MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
 		   $ErrorFound = 1;
@@ -539,13 +544,24 @@ sub new {
           $InSubject = 0;
         }
       }
-      if (!$IPFound && $recdata =~ /^Received: .+\[(\d+\.\d+\.\d+\.\d+)\]/i) {
-        $message->{clientip} = $1;
-        $IPFound = 1;
-      } elsif (!$IPFound && $recdata =~ /^Received: .+\[([\dabcdef.:]+)\]/i) {
-        # It is an IPv6 address
-        $message->{clientip} = $1;
-        $IPFound = 1;
+      if ($recdata =~ /^Received: .+\[(\d+\.\d+\.\d+\.\d+)\]/i) {
+        unless ($read1strcvd) {
+          $ipfromheader = $1;
+          $read1strcvd = 1;
+        }
+        unless ($IPFound) {
+          $message->{clientip} = $1;
+          $IPFound = 1;
+        }
+      } elsif ($recdata =~ /^Received: .+\[([\dabcdef.:]+)\]/i) {
+        unless ($read1strcvd) {
+          $ipfromheader = $1;
+          $read1strcvd = 1;
+        }
+        unless ($IPFound) {
+          $message->{clientip} = $1;
+          $IPFound = 1;
+        }
       } elsif (!$IPFound &&
                $recdata =~ /^Received: .+\(Postfix/i) {
         $message->{clientip} = '127.0.0.1';  #spoof local sender from localhost
@@ -605,7 +621,8 @@ sub new {
 	       @npos = ();
 	    } else {
 	      seek $RQf, $recdata+0, 0; # jump back or forward. we should chk this works.
-	      if ($recdata+0 < $npos[$#npos-1]) {
+              if ($recdata+0 < $MsgContSize+$DataOffset) {
+	      #20090216 if ($recdata+0 < $npos[$#npos-1]) {
 	         if ($recdata+0 >= $npos[0]) {
 		    MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
 		    $ErrorFound = 1;
@@ -631,6 +648,7 @@ sub new {
         $message->{deleted} = 1;
         $message->{gonefromdisk} = 1; # Don't try to delete the original
         $message->{store}->Unlock(); # Unlock it so other processes can pick it up
+        $message->{abandoned} = 1; # JKF 20090301 This message was scrapped
 
         return 0;
       }
@@ -763,6 +781,9 @@ sub new {
     # If we didn't find an IP address, then put in 0.0.0.0 so that at least
     # we have something there
     $message->{clientip} = '0.0.0.0' unless $IPFound;
+    # If we were told to get the IP address from the headers, and there was one
+    $message->{clientip} = $ipfromheader
+      if $getipfromheader && $read1strcvd && $ipfromheader ne "";
 
     # Decode ISO subject lines into UTF8
     # Needed for UTF8 support in MailWatch 2.0
@@ -804,6 +825,7 @@ sub new {
       $message->{deleted} = 1;
       $message->{gonefromdisk} = 1; # Don't try to delete the original
       $message->{store}->Unlock(); # Unlock it so other processes can pick it up
+      $message->{abandoned} = 1; # Retry this message as it was ditched
 
       return 0;
     }
@@ -905,8 +927,20 @@ sub new {
 
     # Find the X record
     my($pos);
-    $pos = $#{$message->{metadata}};
-    $pos-- while ($pos >= 0 && $message->{metadata}[$pos] !~ /^X/);
+
+    # DKIM: This is for adding the header at the position of the first N record
+    # DKIM: Find the first N (or else the X as a safeguard)
+    if ($message->{newheadersattop}) {
+      $pos = 0;
+      while ($pos < $#{$message->{metadata}} &&
+             $message->{metadata}[$pos] !~ /^[NX]/) {
+        $pos++;
+      }
+    } else {
+      # DKIM: This is for adding the header at the end, pos-->X record
+      $pos = $#{$message->{metadata}};
+      $pos-- while ($pos >= 0 && $message->{metadata}[$pos] !~ /^X/);
+    }
     #print STDERR "*** AddHeader $newkey $newvalue at position $pos\n";
 
     # Need to split the new header data into the 1st line and a list of
@@ -929,14 +963,26 @@ sub new {
   sub DeleteHeader {
     my($this, $message, $key) = @_;
 
-    my($pos);
+    my $usingregexp = ($key =~ s/^\/(.*)\/$/$1/)?1:0;
+
+    # Add a colon if they forgot it.
+    $key .= ':' unless $usingregexp || $key =~ /:$/;
+    # If it's not a regexp, then anchor it and sanitise it.
+    $key = '^' . quotemeta($key) unless $usingregexp;
+
+    my($pos, $line);
     $pos = 0;
     $pos++ while ($message->{metadata}[$pos] !~ /^M/);
     # Now points at the M record
     $pos++;
     # Now points at first N record
     while ($pos < @{$message->{metadata}}) {
-      if ($message->{metadata}[$pos] =~ /^N$key/i) {
+      $line = $message->{metadata}[$pos];
+      if ($line =~ s/^N//) {
+        unless ($line =~ /$key/i) {
+          $pos++;
+          next;
+        }
         # We have found the start of 1 occurrence of this header
         splice @{$message->{metadata}}, $pos, 1;
         # Delete continuation lines
@@ -981,7 +1027,8 @@ sub new {
   sub ReplaceHeader {
     my($this, $message, $key, $newvalue) = @_;
 
-    $this->DeleteHeader($message, $key);
+    # DKIM: Don't do DeleteHeader if adding headers at top
+    $this->DeleteHeader($message, $key) unless $message->{dkimfriendly};
     $this->AddHeader($message, $key, $newvalue);
   }
 
@@ -1475,7 +1522,7 @@ sub FindHashDirDepth {
     my($batch, $onlyid) = @_;
 
     my($queuedirname, $queuedir, $queue1dir, $queue2dir, $MsgsInQueue);
-    my($DirtyMsgs, $DirtyBytes, $CleanMsgs, $CleanBytes);
+    my($getipfromheader, $DirtyMsgs, $DirtyBytes, $CleanMsgs, $CleanBytes);
     my($HitLimit1, $HitLimit2, $HitLimit3, $HitLimit4);
     my($MaxCleanB, $MaxCleanM, $MaxDirtyB, $MaxDirtyM);
     my(%ModDate, $mta, $file, $file1, $file2, $tmpdate);
@@ -1488,6 +1535,7 @@ sub FindHashDirDepth {
     $queue2dir = new DirHandle;
     $MsgsInQueue = 0;
     $delay     = MailScanner::Config::Value('queuescaninterval');
+    $getipfromheader = MailScanner::Config::Value('getipfromheader');
     #print STDERR "Inq = " . $global::MS->{inq} . "\n";
     #print STDERR "dir = " . $global::MS->{inq}{dir} . "\n";
     @queuedirnames = @{$global::MS->{inq}{dir}};
@@ -1632,6 +1680,7 @@ sub FindHashDirDepth {
       }
 
       $batchempty = 1;
+      my $maxattempts = MailScanner::Config::Value('procdbattempts');
 
       # Keep going until end of dir or have reached every imposed limit. This
       # now processes the files oldest first to make for fairer queue cleanups.
@@ -1681,7 +1730,8 @@ sub FindHashDirDepth {
         #print STDERR "Adding $id to batch\n";
 
         # Lock and read the qf file. Skip this message if the lock fails.
-        $newmessage = MailScanner::Message->new($id, $queuedirname);
+        $newmessage = MailScanner::Message->new($id, $queuedirname,
+                                                $getipfromheader);
         if ($newmessage eq 'INVALID') {
           #$invalidfiles .= "$id ";
           $invalidfiles .= "$idorig ";
@@ -1689,6 +1739,59 @@ sub FindHashDirDepth {
         }
         next unless $newmessage;
 
+        # JKF 20090301 Skip this message if $id has been scanned
+        # too many times.
+        # JKF 20090301 Read the number of times this message id has
+        # been processed. If over the limit, then ignore it.
+        # JKF 20090301 Should ideally make it in a batch of its own,
+        # and try that once or twice, *then* skip it. But let's try
+        # a simple version first.
+        # Just do a "next" if we want to skip the message.
+        if ($maxattempts) {
+          my $nexttime = time + 120 + int(rand(240)); # 4 +- 2 minutes
+          #my $nexttime = time + 10 + int(rand(20)); # 4 +- 2 minutes
+          my @attempts = $MailScanner::ProcDBH->selectrow_array(
+           "SELECT id,count,nexttime FROM processing WHERE (id='" . $id . "')");
+          #print STDERR "id       = \"$attempts[0]\"\n";
+          #print STDERR "count    = \"$attempts[1]\"\n";
+          #print STDERR "nexttime = \"$attempts[2]\"\n";
+          if (@attempts && $attempts[1]>=$maxattempts) {
+            MailScanner::Log::WarnLog("Warning: skipping message %s as it has been attempted too many times", $id);
+            # JKF 20090301 next;
+            # Instead of just skipping it, quarantine it and notify
+            # the local postmaster.
+            $newmessage->QuarantineDOS();
+            #print STDERR "Moving $attempts[0], $attempts[1], $attempts[2] into archive table\n";
+            $MailScanner::ProcDBH->do(
+              "DELETE FROM processing WHERE id='" . $id . "'");
+            $MailScanner::ProcDBH->do(
+              "INSERT INTO archive (id, count, nexttime) " .
+              "VALUES ('$attempts[0]', $attempts[1], $attempts[2])");
+          } elsif (defined $attempts[1]) {
+            # We have tried this message before
+            if (time>=$attempts[2]) {
+              # Time for next attempt has arrived
+              $MailScanner::ProcDBH->do(
+                "UPDATE processing SET count=count+1, nexttime=$nexttime " .
+                " WHERE id='" . $id . "'");
+              MailScanner::Log::InfoLog(
+                "Making attempt %d at processing message %s",
+                $attempts[1]+1, $id);
+              #print STDERR "Incremented $id\n";
+            } else {
+              # Not time for next attempt yet, so ignore the message
+              $newmessage->DropFromBatch();
+              next;
+            }
+          } else {
+            # We have never seen this message before
+            $MailScanner::ProcDBH->do(
+              "INSERT INTO processing (id, count, nexttime) " .
+              "VALUES ('" . $id . "', 1, $nexttime)");
+            #print STDERR "Inserted $id\n";
+          }
+        }
+
         $batch->{messages}{"$id"} = $newmessage;
         $newmessage->{mtime} = $mtime;
         #print STDERR "Added $id to batch\n";
